#ifndef RdbStats_H
#define RdbStats_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of rdbstats
 *
 * rdbstats is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * rdbstats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <rdbxx/RDB.h>
#include <cmath>                        // sqrt fabs

using namespace std;

/**
 * The base class to calculate :
 * average, maximum, minimum, num, stddev and sum.
 * The derived classes are: RdbStatsPercentile, RdbStatsQuartile.
 */
class RdbStats {

public:

  virtual ~RdbStats( ) throw( );

  RdbStats( RDB& irdbtable, const string& name ) throw( Exception );

  /// Perform the final statistic for the set.
  virtual int calculate_statistics( ) throw( );

  /// To get the absolute value of the average.
  double get_absolute_average( ) const { return fabs( the_statistics[AVG] ); }

  virtual void init( ) throw( );

  virtual void normalize_results( const double norm ) throw( );

  virtual void set_output_columns( RDB& ordbtable ) throw( Exception );

  /// Read the column from RDB++, update the statistics for the column.
  virtual void update_statistics( ) throw( Exception );

private:

  // input_column is the input rdb column ptr. It is
  // allocated by the library so do not free the memory.
  RDBColumn* input_column;

  // The pointer to the output rdb columns.
  vector<RDBColumn*> output_stats;

protected:

  enum Stats { AVG, MAX, MIN, SD, SUM, SUM2, SUM_T };

  double   num_n;

  string column_name;

  double the_statistics[ 7 ];

  double get_value( ) throw( Exception );

};

#endif
