#ifndef RdbStatsPercentiles_H
#define RdbStatsPercentiles_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of rdbstats
 *
 * rdbstats is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * rdbstats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <ostream>
#include <vector>

#include "RdbStats.h"

using namespace std;

/**
 * A class to calculate the percentiles, RdbStatsPercentiles isA 
 * special case of the class RdbStats.
 */
class RdbStatsPercentiles : public RdbStats {

public:

  ~RdbStatsPercentiles( ) throw( ) {

    delete [] the_percentile_result;
    the_percentile_result = NULL;

  }

  // The user has specified the percentile to use:
  /// --percentiles 12,34.. and --quartile
  RdbStatsPercentiles( RDB& irdbtable, const string& name,
		       const string& percent_list, bool implicitquartile )
    throw( Exception );

  // --quartiles
  RdbStatsPercentiles( RDB& irdbtable, const string& name ) throw( Exception );

  virtual int calculate_statistics( ) throw( );

  void init( ) throw( );

  virtual void normalize_results( const double norm ) throw( );

  void set_output_columns( RDB& ordbtable ) throw( Exception );

  virtual void update_statistics( ) throw( Exception );

protected:

  vector< double > data;

  vector< pair< string, double > > the_percentile;

  double the_median;

  double* the_percentile_result;

private:

  /// To calculate the median, interpolate if the size of data is even.
  double calculate_median( ) throw( );

  /// It should interpolate like calculate_median
  double calculate_percentile( const double percentile ) throw( );

};

#endif
