#ifndef SelectedCols_H
#define SelectedCols_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of rdbstats
 *
 * rdbstats is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * rdbstats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <vector>

#include <suplibxx/str.h>
#include <Exception/Exception.h>
#include <rdbxx/RDB.h>

#include "Options.h"

/**
 *
 * A class to figure out the name of the columns
 * for which the statistics are to be calculated.
 * The users can select all column by specifying
 * with the --all option.
 * The user can specify the selected columns by
 * listing the column names in conjunction with
 * the -d option to override the column definition.
 *
 */
using namespace std;

class SelectedCols {

  friend ostream& operator << ( ostream& os, SelectedCols& a ) {
    a.print( os );
    return os;
  }

  friend ostream& operator << ( ostream& os, SelectedCols* a ) {
    a->print( os );
    return os;
  }

public:

  /// The user has specified selected columns
  SelectedCols( const vector< string >& cols, RDB& rdbtable,
		clo::parser& clo ) throw( Exception );

  /// The --all option has been invoked.
  SelectedCols( RDB& rdbtable, clo::parser& clo ) throw( Exception );

  const vector< string >& get_selected_cols( ) const { return selected_cols; }

  void print( ostream& os ) const;

protected:

  // Contains the names of the columns to calculate the statistics.
  vector< string > selected_cols;

private:

  /// Return true if colname is numeric, return false otherwise.
  bool is_column_numeric( const string& colname, RDB& rdbtable )
    throw( Exception );

  /// Return
  bool is_column_numeric( const string& colname, RDB& rdbtable,
			  const clo::options& options ) throw ( Exception );

  bool is_groupie( const string& name, const vector<string>& groupies );

  void toggle_column_definition( const string& colname, RDB& rdbtable )
    const throw ( Exception );

};

#endif
