// File:  StatsResult.cc

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of rdbstats
//
// rdbstats is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// rdbstats is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#define StatsResult_cc
#include "StatsResult.h"

template <class Type>
StatsResult<Type>::StatsResult( Type& col, RDB& otable ) throw( Exception ) :
  rdb_stats_columns( col ), ordbtable( otable ) {

  try {

#ifdef TRACEFCT
    TraceFct tf( "StatsResult<Type>::StatsResult( Type&, RDB& ) "
		 "throw( Exception )" );
#endif

    // set_output_columns must be done here instead of within
    // the constructor of RdbStats is because stl calls the
    // ~RdbStats upon inserting an element in the vector container
    // which will lead the pointers of RdbStats to point to no man's land.
    for ( size_t ii = 0; ii < rdb_stats_columns.size( ); ii++ )
      rdb_stats_columns[ ii ].set_output_columns( ordbtable );

  } catch( Exception& e ) {

    throw;

  }

}

template <class Type>
void StatsResult<Type>::calculate_statistics( ) throw( ) {

#ifdef TRACEFCT
    TraceFct tf( "StatsResult<Type>::calculate_statistics(  ) throw( )" );
#endif

  for ( size_t ii = 0; ii < rdb_stats_columns.size( ); ii++ )
      rdb_stats_columns[ ii ].calculate_statistics( ) ;

  //  ordbtable.advanceIdx( );
  ordbtable.write( );

  for ( size_t ii = 0; ii < rdb_stats_columns.size( ); ii++ )
    rdb_stats_columns[ ii ].init( );

}

template <class Type>
void StatsResult<Type>::update_statistics( ) throw( Exception ) {

  try {

#ifdef TRACEFCT
    TraceFct tf( "StatsResult<Type>::update_statistics(  ) throw( )" );
#endif

    for ( size_t ii = 0; ii < rdb_stats_columns.size( ); ii++ )
      rdb_stats_columns[ ii ].update_statistics( );

  } catch ( Exception& e ) {

    throw;

  }

}
